body{margin:0;font-family:-apple-system,BlinkMacSystemFont,"Segoe UI","Roboto","Oxygen","Ubuntu","Cantarell","Fira Sans","Droid Sans","Helvetica Neue",sans-serif;-webkit-font-smoothing:antialiased;-moz-osx-font-smoothing:grayscale}code{font-family:source-code-pro,Menlo,Monaco,Consolas,"Courier New",monospace}.App{text-align:center}#navbar-brand{flex-grow:1;text-align:left}#navbar span{color:#fff}#login-container{max-width:360px;margin:5% auto;text-align:left}#submit-btn{margin-top:20px;margin-bottom:30px}#error-message{color:red}#worklist-container{max-width:850px;margin:20px auto;padding-left:10px;padding-right:10px}#update-time{text-align:left;margin-top:20px}#refersh-btn{position:fixed;right:20px;bottom:50px}#refersh-progress{position:fixed;right:14px;bottom:44px;z-index:1}
/*# sourceMappingURL=main.ea0700e7.chunk.css.map */